@@ -219,6 +219,17 @@ struct ivi_layout_interface {
 	 */
 	void (*add_listener_configure_desktop_surface)(struct wl_listener *listener);
 
+	/**
+	 * \brief add a listener for desktop_surface ping timeout notification
+	 *
+	 * When a desktop_client fails to respond to compositor pings within
+	 * the configured timeout period, this signal is emitted to registered
+	 * listeners. The void* data argument will contain a pointer to the
+	 * weston_desktop_client associated with the unresponsive surface.
+	 */
+
+	void (*add_listener_desktop_surface_ping_timeout)(struct wl_listener *listener);
+
 	/**
 	 * \brief Get all ivi_surfaces which are currently registered and managed
 	 * by the services
@@ -469,6 +480,9 @@ struct ivi_layout_interface {
 
 	/**
 	 * \brief Add a ivi_surface to a ivi_layer which is currently managed by the service
+	 *
+	 * The surface buffer must be committed before the ivi-controller adds
+	 * the ivi_surface to a ivi_layer
 	 */
 	void (*layer_add_surface)(struct ivi_layout_layer *ivilayer,
 				     struct ivi_layout_surface *addsurf);
@@ -633,6 +647,12 @@ struct ivi_layout_interface {
 	 * See add_listener_show_input_panel for more details.
 	 */
 	void (*add_listener_update_input_panel)(struct wl_listener *listener);
+
+	/**
+	 * \brief Set weston_output ready to be painted
+	 */
+	void (*screen_ready)(struct weston_output *output);
+
 };
 
 static inline const struct ivi_layout_interface *
