@@ -56,6 +56,7 @@
 #include "pixel-formats.h"
 #include "pixman-renderer.h"
 #include "renderer-gl/gl-renderer.h"
+#include "renderer-vulkan/vulkan-renderer.h"
 #include "shared/weston-egl-ext.h"
 
 #define DEFAULT_AXIS_STEP_DISTANCE 10
@@ -108,9 +109,18 @@ struct vnc_head {
 	struct weston_head base;
 };
 
+struct vnc_buffer {
+	weston_renderbuffer_t rb;
+	struct nvnc_fb *fb;
+	struct vnc_output *output;
+};
+
 static void
 vnc_output_destroy(struct weston_output *base);
 
+static void
+vnc_buffer_destroy(struct vnc_buffer *buffer);
+
 static inline struct vnc_output *
 to_vnc_output(struct weston_output *base)
 {
@@ -498,7 +508,7 @@ vnc_client_cleanup(struct nvnc_client *client)
 	free(peer);
 	weston_log("VNC Client disconnected\n");
 
-	if (wl_list_empty(&output->peers))
+	if (output && wl_list_empty(&output->peers))
 		weston_output_power_off(&output->base);
 }
 
@@ -539,7 +549,6 @@ vnc_output_update_cursor(struct vnc_output *output)
 	struct weston_buffer *buffer;
 	struct weston_surface *cursor_surface;
 	struct nvnc_fb *fb;
-	int32_t stride;
 	uint8_t *src, *dst;
 	int i;
 
@@ -556,8 +565,6 @@ vnc_output_update_cursor(struct vnc_output *output)
 	cursor_surface = output->cursor_surface;
 	buffer = cursor_surface->buffer_ref.buffer;
 
-	stride = wl_shm_buffer_get_stride(buffer->shm_buffer);
-
 	fb = nvnc_fb_new(buffer->width, buffer->height, DRM_FORMAT_ARGB8888,
 			 buffer->width);
 	assert(fb);
@@ -567,7 +574,7 @@ vnc_output_update_cursor(struct vnc_output *output)
 
 	wl_shm_buffer_begin_access(buffer->shm_buffer);
 	for (i = 0; i < buffer->height; i++)
-		memcpy(dst + i * 4 * buffer->width, src + i * stride,
+		memcpy(dst + i * 4 * buffer->width, src + i * buffer->stride,
 		       4 * buffer->width);
 	wl_shm_buffer_end_access(buffer->shm_buffer);
 
@@ -656,8 +663,7 @@ vnc_log_scope_print_region(struct weston_log_scope *log, pixman_region32_t *regi
 }
 
 static void
-vnc_log_damage(struct vnc_backend *backend, pixman_region32_t *buffer_damage,
-	       pixman_region32_t *update_damage)
+vnc_log_damage(struct vnc_backend *backend, pixman_region32_t *damage)
 {
 	char timestr[128];
 
@@ -666,15 +672,51 @@ vnc_log_damage(struct vnc_backend *backend, pixman_region32_t *buffer_damage,
 
 	weston_log_scope_timestamp(backend->debug, timestr, sizeof timestr);
 
-	weston_log_scope_printf(backend->debug, "%s buffer damage:", timestr);
-	vnc_log_scope_print_region(backend->debug, buffer_damage);
-	weston_log_scope_printf(backend->debug, "\n");
-
-	weston_log_scope_printf(backend->debug, "%s update damage:", timestr);
-	vnc_log_scope_print_region(backend->debug, update_damage);
+	weston_log_scope_printf(backend->debug, "%s damage:", timestr);
+	vnc_log_scope_print_region(backend->debug, damage);
 	weston_log_scope_printf(backend->debug, "\n\n");
 }
 
+static bool
+vnc_rb_discarded_cb(weston_renderbuffer_t rb, void *data)
+{
+	struct vnc_buffer *buffer = (struct vnc_buffer *) data;
+
+	assert(nvnc_get_userdata(buffer->fb) == buffer);
+
+	nvnc_set_userdata(buffer->fb, NULL, NULL);
+	vnc_buffer_destroy(buffer);
+
+	return true;
+}
+
+static struct vnc_buffer *
+vnc_buffer_create(struct nvnc_fb* fb, struct vnc_output *output)
+{
+	const struct pixel_format_info *pfmt =
+		pixel_format_get_info(DRM_FORMAT_XRGB8888);
+	struct weston_renderer *rdr = output->base.compositor->renderer;
+	struct vnc_buffer *buffer = xmalloc(sizeof *buffer);
+
+	buffer->rb = rdr->create_renderbuffer(&output->base, pfmt,
+					      nvnc_fb_get_addr(fb),
+					      output->base.current_mode->width * 4,
+					      vnc_rb_discarded_cb, buffer);
+	buffer->fb = fb;
+	buffer->output = output;
+
+	return buffer;
+}
+
+static void
+vnc_buffer_destroy(struct vnc_buffer *buffer)
+{
+	struct weston_renderer *rdr = buffer->output->base.compositor->renderer;
+
+	rdr->destroy_renderbuffer(buffer->rb);
+	free(buffer);
+}
+
 static void
 vnc_update_buffer(struct nvnc_display *display, struct pixman_region32 *damage)
 {
@@ -682,7 +724,7 @@ vnc_update_buffer(struct nvnc_display *display, struct pixman_region32 *damage)
 	struct vnc_backend *backend = nvnc_get_userdata(server);
 	struct vnc_output *output = backend->output;
 	struct weston_compositor *ec = output->base.compositor;
-	struct weston_renderbuffer *renderbuffer;
+	struct vnc_buffer *buffer;
 	pixman_region32_t local_damage;
 	pixman_region16_t nvnc_damage;
 	struct nvnc_fb *fb;
@@ -690,49 +732,16 @@ vnc_update_buffer(struct nvnc_display *display, struct pixman_region32 *damage)
 	fb = nvnc_fb_pool_acquire(output->fb_pool);
 	assert(fb);
 
-	renderbuffer = nvnc_get_userdata(fb);
-	if (!renderbuffer) {
-		const struct pixel_format_info *pfmt;
-
-		pfmt = pixel_format_get_info(DRM_FORMAT_XRGB8888);
-
-		switch (ec->renderer->type) {
-		case WESTON_RENDERER_PIXMAN: {
-			const struct pixman_renderer_interface *pixman;
-
-			pixman = ec->renderer->pixman;
-
-			renderbuffer =
-				pixman->create_image_from_ptr(&output->base, pfmt,
-							      output->base.width,
-							      output->base.height,
-							      nvnc_fb_get_addr(fb),
-							      output->base.width * 4);
-			break;
-		}
-		case WESTON_RENDERER_GL: {
-			renderbuffer =
-				ec->renderer->gl->create_fbo(&output->base, pfmt,
-							     output->base.width,
-							     output->base.height,
-							     nvnc_fb_get_addr(fb));
-			break;
-		}
-		default:
-			unreachable("cannot have auto renderer at runtime");
-		}
-
-		/* This is a new buffer, so the whole surface is damaged. */
-		pixman_region32_copy(&renderbuffer->damage,
-				     &output->base.region);
-
-		nvnc_set_userdata(fb, renderbuffer,
-				  (nvnc_cleanup_fn)weston_renderbuffer_unref);
+	buffer = nvnc_get_userdata(fb);
+	if (!buffer) {
+		buffer = vnc_buffer_create(fb, output);
+		nvnc_set_userdata(fb, buffer,
+				  (nvnc_cleanup_fn) vnc_buffer_destroy);
 	}
 
-	vnc_log_damage(backend, &renderbuffer->damage, damage);
+	vnc_log_damage(backend, damage);
 
-	ec->renderer->repaint_output(&output->base, damage, renderbuffer);
+	ec->renderer->repaint_output(&output->base, damage, buffer->rb);
 
 	/* Convert to local coordinates */
 	pixman_region32_init(&local_damage);
@@ -812,8 +821,8 @@ vnc_output_enable(struct weston_output *base)
 	case WESTON_RENDERER_PIXMAN: {
 		const struct pixman_renderer_output_options options = {
 			.fb_size = {
-				.width = output->base.width,
-				.height = output->base.height,
+				.width = output->base.current_mode->width,
+				.height = output->base.current_mode->height,
 			},
 			.format = backend->formats[0],
 		};
@@ -824,18 +833,33 @@ vnc_output_enable(struct weston_output *base)
 	case WESTON_RENDERER_GL: {
 		const struct gl_renderer_fbo_options options = {
 			.area = {
-				.width = output->base.width,
-				.height = output->base.height,
+				.width = output->base.current_mode->width,
+				.height = output->base.current_mode->height,
 			},
 			.fb_size = {
-				.width = output->base.width,
-				.height = output->base.height,
+				.width = output->base.current_mode->width,
+				.height = output->base.current_mode->height,
 			},
 		};
 		if (renderer->gl->output_fbo_create(&output->base, &options) < 0)
 			return -1;
 		break;
 	}
+	case WESTON_RENDERER_VULKAN: {
+		const struct vulkan_renderer_surfaceless_options options = {
+			.area = {
+				.width = output->base.current_mode->width,
+				.height = output->base.current_mode->height,
+			},
+			.fb_size = {
+				.width = output->base.current_mode->width,
+				.height = output->base.current_mode->height,
+			},
+		};
+		if (renderer->vulkan->output_surfaceless_create(&output->base, &options) < 0)
+			return -1;
+		break;
+	}
 	default:
 		unreachable("cannot have auto renderer at runtime");
 	}
@@ -845,10 +869,10 @@ vnc_output_enable(struct weston_output *base)
 							     finish_frame_handler,
 							     output);
 
-	output->fb_pool = nvnc_fb_pool_new(output->base.width,
-					   output->base.height,
+	output->fb_pool = nvnc_fb_pool_new(output->base.current_mode->width,
+					   output->base.current_mode->height,
 					   backend->formats[0]->format,
-					   output->base.width);
+					   output->base.current_mode->width);
 
 	output->display = nvnc_display_new(0, 0);
 
@@ -871,6 +895,7 @@ vnc_output_disable(struct weston_output *base)
 	if (!output->base.enabled)
 		return 0;
 
+	nvnc_remove_display(backend->server, output->display);
 	nvnc_display_unref(output->display);
 	nvnc_fb_pool_unref(output->fb_pool);
 
@@ -881,6 +906,9 @@ vnc_output_disable(struct weston_output *base)
 	case WESTON_RENDERER_GL:
 		renderer->gl->output_destroy(&output->base);
 		break;
+	case WESTON_RENDERER_VULKAN:
+		renderer->vulkan->output_destroy(&output->base);
+		break;
 	default:
 		unreachable("cannot have auto renderer at runtime");
 	}
@@ -931,14 +959,6 @@ vnc_create_output(struct weston_backend *backend, const char *name)
 	return &output->base;
 }
 
-static void
-vnc_shutdown(struct weston_backend *base)
-{
-	struct vnc_backend *backend = container_of(base, struct vnc_backend, base);
-
-	nvnc_close(backend->server);
-}
-
 static void
 vnc_destroy(struct weston_backend *base)
 {
@@ -946,6 +966,8 @@ vnc_destroy(struct weston_backend *base)
 	struct weston_compositor *ec = backend->compositor;
 	struct weston_head *head, *next;
 
+	nvnc_close(backend->server);
+
 	wl_list_remove(&backend->base.link);
 
 	wl_event_source_remove(backend->aml_event);
@@ -1060,6 +1082,9 @@ vnc_output_assign_planes(struct weston_output *base)
 
 	assert(output);
 
+	if (output->base.disable_planes)
+		return;
+
 	if (wl_list_empty(&output->peers))
 		return;
 
@@ -1068,6 +1093,19 @@ vnc_output_assign_planes(struct weston_output *base)
 		vnc_output_assign_cursor_plane(output);
 }
 
+static void
+vnc_set_dpms(struct weston_output *base, enum dpms_enum level)
+{
+	struct vnc_output *output = to_vnc_output(base);
+
+	if (level == WESTON_DPMS_ON)
+		weston_output_schedule_repaint(base);
+	else if (output->base.repaint_status == REPAINT_AWAITING_COMPLETION) {
+		wl_event_source_timer_update(output->finish_frame_timer, 0);
+		weston_output_schedule_repaint_reset(base);
+	}
+}
+
 static int
 vnc_switch_mode(struct weston_output *base, struct weston_mode *target_mode)
 {
@@ -1081,7 +1119,14 @@ vnc_switch_mode(struct weston_output *base, struct weston_mode *target_mode)
 	fb_size.width = target_mode->width;
 	fb_size.height = target_mode->height;
 
-	weston_renderer_resize_output(base, &fb_size, NULL);
+	/* vnc_buffers are stored as user data pointers into the renderbuffers
+	 * for the discarded callback. weston_renderer_resize_output(), which
+	 * triggers the renderbuffer's discarded callbacks, must be called
+	 * before nvnc_fb_pool_resize(), which destroys all the nvnc_fbs and
+	 * their associated vnc_buffers, so that the vnc_buffers are valid at
+	 * callback. */
+	if (!weston_renderer_resize_output(base, &fb_size, NULL))
+		return -1;
 
 	nvnc_fb_pool_resize(output->fb_pool, target_mode->width,
 			    target_mode->height, DRM_FORMAT_XRGB8888,
@@ -1113,7 +1158,7 @@ vnc_output_set_size(struct weston_output *base, int width, int height,
 	output->base.repaint = vnc_output_repaint;
 	output->base.assign_planes = vnc_output_assign_planes;
 	output->base.set_backlight = NULL;
-	output->base.set_dpms = NULL;
+	output->base.set_dpms = vnc_set_dpms;
 	output->base.switch_mode = vnc_switch_mode;
 
 	output->resizeable = resizeable;
@@ -1153,7 +1198,6 @@ vnc_backend_create(struct weston_compositor *compositor,
 	wl_list_init(&backend->base.link);
 
 	backend->compositor = compositor;
-	backend->base.shutdown = vnc_shutdown;
 	backend->base.destroy = vnc_destroy;
 	backend->base.create_output = vnc_create_output;
 	backend->vnc_monitor_refresh_rate = config->refresh_rate * 1000;
@@ -1192,6 +1236,17 @@ vnc_backend_create(struct weston_compositor *compositor,
 				goto err_compositor;
 			break;
 		}
+		case WESTON_RENDERER_VULKAN: {
+			const struct vulkan_renderer_display_options options = {
+				.formats = backend->formats,
+				.formats_count = backend->formats_count,
+			};
+			if (weston_compositor_init_renderer(compositor,
+							    WESTON_RENDERER_VULKAN,
+							    &options.base) < 0)
+				goto err_compositor;
+			break;
+		}
 		default:
 			weston_log("Unsupported renderer requested\n");
 			goto err_compositor;
@@ -1235,40 +1290,57 @@ vnc_backend_create(struct weston_compositor *compositor,
 	nvnc_set_userdata(backend->server, backend, NULL);
 	nvnc_set_name(backend->server, "Weston VNC backend");
 
-	if (!nvnc_has_auth()) {
-		weston_log("Neat VNC built without TLS support\n");
-		goto err_output;
-	}
-	if (!config->server_cert && !config->server_key) {
-		weston_log("The VNC backend requires a key and a certificate for TLS security"
-			   " (--vnc-tls-cert/--vnc-tls-key)\n");
-		goto err_output;
-	}
-	if (!config->server_cert) {
-		weston_log("Missing TLS certificate (--vnc-tls-cert)\n");
-		goto err_output;
-	}
-	if (!config->server_key) {
-		weston_log("Missing TLS key (--vnc-tls-key)\n");
-		goto err_output;
-	}
+	if (!config->disable_tls) {
+		if (!nvnc_has_auth()) {
+			weston_log("Neat VNC built without TLS support\n");
+			goto err_output;
+		}
+		if (!config->server_cert && !config->server_key) {
+			weston_log(
+				"The VNC backend requires a key and a "
+				"certificate for TLS security"
+				" (--vnc-tls-cert/--vnc-tls-key)\n");
+			goto err_output;
+		}
+		if (!config->server_cert) {
+			weston_log(
+				"Missing TLS certificate (--vnc-tls-cert)\n");
+			goto err_output;
+		}
+		if (!config->server_key) {
+			weston_log("Missing TLS key (--vnc-tls-key)\n");
+			goto err_output;
+		}
 
-	ret = nvnc_set_tls_creds(backend->server, config->server_key,
-				 config->server_cert);
-	if (ret) {
-		weston_log("Failed set TLS credentials\n");
-		goto err_output;
-	}
+		ret = nvnc_set_tls_creds(backend->server, config->server_key,
+					 config->server_cert);
+		if (ret) {
+			weston_log("Failed set TLS credentials\n");
+			goto err_output;
+		}
 
-	ret = nvnc_enable_auth(backend->server, NVNC_AUTH_REQUIRE_AUTH |
-			       NVNC_AUTH_REQUIRE_ENCRYPTION, vnc_handle_auth,
-			       NULL);
-	if (ret) {
-		weston_log("Failed to enable TLS support\n");
-		goto err_output;
-	}
+		ret = nvnc_enable_auth(
+			backend->server,
+			NVNC_AUTH_REQUIRE_AUTH | NVNC_AUTH_REQUIRE_ENCRYPTION,
+			vnc_handle_auth, NULL);
+		if (ret) {
+			weston_log("Failed to enable TLS support\n");
+			goto err_output;
+		}
 
-	weston_log("TLS support activated\n");
+		weston_log("TLS support activated\n");
+	} else {
+		ret = nvnc_enable_auth(backend->server, NVNC_AUTH_REQUIRE_AUTH,
+				       vnc_handle_auth, NULL);
+		if (ret) {
+			weston_log("Failed to enable authentication\n");
+			goto err_output;
+		}
+
+		weston_log(
+			"warning: VNC enabled without Transport Layer "
+			"Security!\n");
+	}
 
 	ret = weston_plugin_api_register(compositor, WESTON_VNC_OUTPUT_API_NAME,
 					 &api, sizeof(api));
@@ -1316,6 +1388,7 @@ weston_backend_init(struct weston_compositor *compositor,
 		switch (compositor->renderer->type) {
 		case WESTON_RENDERER_PIXMAN:
 		case WESTON_RENDERER_GL:
+		case WESTON_RENDERER_VULKAN:
 			break;
 		default:
 			weston_log("Renderer not supported by VNC backend\n");
