@@ -37,6 +37,7 @@
 
 #include <libweston/desktop.h>
 #include "internal.h"
+#include "libweston-internal.h"
 #include "shared/helpers.h"
 
 /************************************************************************************
@@ -74,6 +75,7 @@ struct weston_desktop_xdg_surface {
 	struct weston_desktop *desktop;
 	struct weston_surface *surface;
 	struct weston_desktop_surface *desktop_surface;
+	bool committed;
 	bool configured;
 	struct wl_event_source *configure_idle;
 	struct wl_list configure_list; /* weston_desktop_xdg_surface_configure::link */
@@ -127,7 +129,6 @@ struct weston_desktop_xdg_popup {
 	struct weston_desktop_xdg_surface base;
 
 	struct wl_resource *resource;
-	bool committed;
 	struct weston_desktop_xdg_surface *parent;
 	struct weston_desktop_seat *seat;
 	struct weston_geometry geometry;
@@ -141,11 +142,40 @@ struct weston_desktop_xdg_popup {
 	    sizeof(struct weston_desktop_xdg_popup))
 #define weston_desktop_surface_configure_biggest_size weston_desktop_surface_role_biggest_size
 
+static bool
+intersect_rectangles(struct weston_geometry *dst,
+		     const struct weston_geometry *src1,
+		     const struct weston_geometry *src2)
+{
+	int dst_x, dst_y;
+	int dst_w, dst_h;
+
+	dst_x = MAX(src1->x, src2->x);
+	dst_y = MAX(src1->y, src2->y);
+	dst_w = MIN(src1->x + src1->width, src2->x + src2->width) - dst_x;
+	dst_h = MIN(src1->y + src1->height, src2->y + src2->height) - dst_y;
+
+	if (dst_w > 0 && dst_h > 0) {
+		*dst = (struct weston_geometry) {
+			.x = dst_x,
+			.y = dst_y,
+			.width = dst_w,
+			.height = dst_h,
+		};
+		return true;
+	} else {
+		*dst = (struct weston_geometry) {
+			.x = 0,
+			.y = 0,
+			.width = 0,
+			.height = 0,
+		};
+		return false;
+	}
+}
 
 static struct weston_geometry
-weston_desktop_xdg_positioner_get_geometry(struct weston_desktop_xdg_positioner *positioner,
-					   struct weston_desktop_surface *dsurface,
-					   struct weston_desktop_surface *parent)
+process_positioner(struct weston_desktop_xdg_positioner *positioner)
 {
 	struct weston_geometry geometry = {
 		.x = positioner->offset.x,
@@ -214,10 +244,328 @@ weston_desktop_xdg_positioner_get_geometry(struct weston_desktop_xdg_positioner
 		geometry.x -= geometry.width / 2;
 	}
 
-	if (positioner->constraint_adjustment == XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_NONE)
+	return geometry;
+}
+
+static void
+positioner_flip_vertically(struct weston_desktop_xdg_positioner *positioner)
+{
+	switch (positioner->anchor) {
+	case XDG_POSITIONER_ANCHOR_TOP:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_BOTTOM;
+		break;
+	case XDG_POSITIONER_ANCHOR_BOTTOM:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_TOP;
+		break;
+	case XDG_POSITIONER_ANCHOR_TOP_LEFT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_BOTTOM_LEFT;
+		break;
+	case XDG_POSITIONER_ANCHOR_BOTTOM_LEFT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_TOP_LEFT;
+		break;
+	case XDG_POSITIONER_ANCHOR_TOP_RIGHT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_BOTTOM_RIGHT;
+		break;
+	case XDG_POSITIONER_ANCHOR_BOTTOM_RIGHT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_TOP_RIGHT;
+		break;
+	case XDG_POSITIONER_ANCHOR_NONE:
+	case XDG_POSITIONER_ANCHOR_LEFT:
+	case XDG_POSITIONER_ANCHOR_RIGHT:
+		break;
+	}
+
+	switch (positioner->gravity) {
+	case XDG_POSITIONER_GRAVITY_TOP:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_BOTTOM;
+		break;
+	case XDG_POSITIONER_GRAVITY_BOTTOM:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_TOP;
+		break;
+	case XDG_POSITIONER_GRAVITY_TOP_LEFT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_BOTTOM_LEFT;
+		break;
+	case XDG_POSITIONER_GRAVITY_BOTTOM_LEFT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_TOP_LEFT;
+		break;
+	case XDG_POSITIONER_GRAVITY_TOP_RIGHT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_BOTTOM_RIGHT;
+		break;
+	case XDG_POSITIONER_GRAVITY_BOTTOM_RIGHT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_TOP_RIGHT;
+		break;
+	case XDG_POSITIONER_GRAVITY_NONE:
+	case XDG_POSITIONER_GRAVITY_LEFT:
+	case XDG_POSITIONER_GRAVITY_RIGHT:
+		break;
+	}
+}
+
+static void
+positioner_flip_horizontally(struct weston_desktop_xdg_positioner *positioner)
+{
+	switch (positioner->anchor) {
+	case XDG_POSITIONER_ANCHOR_NONE:
+	case XDG_POSITIONER_ANCHOR_TOP:
+	case XDG_POSITIONER_ANCHOR_BOTTOM:
+		break;
+	case XDG_POSITIONER_ANCHOR_TOP_LEFT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_TOP_RIGHT;
+		break;
+	case XDG_POSITIONER_ANCHOR_BOTTOM_LEFT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_BOTTOM_RIGHT;
+		break;
+	case XDG_POSITIONER_ANCHOR_TOP_RIGHT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_TOP_LEFT;
+		break;
+	case XDG_POSITIONER_ANCHOR_BOTTOM_RIGHT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_BOTTOM_LEFT;
+		break;
+	case XDG_POSITIONER_ANCHOR_LEFT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_RIGHT;
+		break;
+	case XDG_POSITIONER_ANCHOR_RIGHT:
+		positioner->anchor = XDG_POSITIONER_ANCHOR_LEFT;
+		break;
+	}
+
+	switch (positioner->gravity) {
+	case XDG_POSITIONER_GRAVITY_NONE:
+	case XDG_POSITIONER_GRAVITY_TOP:
+	case XDG_POSITIONER_GRAVITY_BOTTOM:
+		break;
+	case XDG_POSITIONER_GRAVITY_TOP_LEFT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_TOP_RIGHT;
+		break;
+	case XDG_POSITIONER_GRAVITY_BOTTOM_LEFT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_BOTTOM_RIGHT;
+		break;
+	case XDG_POSITIONER_GRAVITY_TOP_RIGHT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_TOP_LEFT;
+		break;
+	case XDG_POSITIONER_GRAVITY_BOTTOM_RIGHT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_BOTTOM_LEFT;
+		break;
+	case XDG_POSITIONER_GRAVITY_LEFT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_RIGHT;
+		break;
+	case XDG_POSITIONER_GRAVITY_RIGHT:
+		positioner->gravity = XDG_POSITIONER_GRAVITY_LEFT;
+		break;
+	}
+}
+
+static bool
+is_constraint_satisfied(struct weston_geometry *surface_geometry,
+			struct weston_geometry *output_geometry,
+			struct weston_geometry *intersection)
+{
+	intersect_rectangles(intersection, surface_geometry, output_geometry);
+	return (intersection->width == surface_geometry->width &&
+		intersection->height == surface_geometry->height);
+}
+
+static void
+local_geometry_to_global(struct weston_geometry *geometry,
+			 struct weston_view *parent_view,
+			 struct weston_geometry *global_geometry)
+{
+	pixman_box32_t geometry_rect;
+	pixman_box32_t global_rect;
+
+	geometry_rect = (pixman_box32_t) {
+		.x1 = geometry->x,
+		.y1 = geometry->y,
+		.x2 = geometry->x + geometry->width,
+		.y2 = geometry->y + geometry->height,
+	};
+	global_rect = weston_matrix_transform_rect(&parent_view->transform.matrix,
+						   geometry_rect);
+	global_geometry->x = global_rect.x1;
+	global_geometry->y = global_rect.y1;
+	global_geometry->width = global_rect.x2 - global_rect.x1;
+	global_geometry->height = global_rect.y2 - global_rect.y1;
+}
+
+static bool
+is_geometry_satisfied(struct weston_view *parent_view,
+		      struct weston_geometry *geometry,
+		      struct weston_output *output,
+		      struct weston_geometry *intersection)
+{
+	struct weston_geometry global_geometry;
+	struct weston_geometry output_geometry;
+	struct weston_geometry global_intersection;
+	struct weston_coord_surface surface_coord;
+	struct weston_coord_global tmp_g;
+	bool is_satisfied;
+
+	local_geometry_to_global(geometry, parent_view, &global_geometry);
+
+	output_geometry = (struct weston_geometry) {
+		.x = output->pos.c.x,
+		.y = output->pos.c.y,
+		.width = output->width,
+		.height = output->height,
+	};
+
+	is_satisfied = is_constraint_satisfied(&global_geometry,
+					       &output_geometry,
+					       &global_intersection);
+
+	tmp_g.c = weston_coord(global_intersection.x,
+			       global_intersection.y);
+	surface_coord = weston_coord_global_to_surface(parent_view, tmp_g);
+	intersection->x = round(surface_coord.c.x);
+	intersection->y = round(surface_coord.c.y);
+
+	tmp_g.c = weston_coord(global_intersection.x +
+			       global_intersection.width,
+			       global_intersection.y +
+			       global_intersection.height);
+	surface_coord = weston_coord_global_to_surface(parent_view, tmp_g);
+
+	intersection->width = round(surface_coord.c.x) - intersection->x;
+	intersection->height = round(surface_coord.c.y) - intersection->y;
+
+	return is_satisfied;
+}
+
+static void
+try_flip_position(struct weston_desktop_surface *surface,
+		  struct weston_view *parent_view,
+		  struct weston_output *output,
+		  struct weston_geometry *geometry,
+		  struct weston_desktop_xdg_positioner *positioner,
+		  enum xdg_positioner_constraint_adjustment constraint_adjustment)
+{
+	struct weston_desktop_xdg_positioner flipped_positioner = *positioner;
+	struct weston_geometry flipped_geometry;
+	struct weston_geometry flipped_intersection;
+
+	switch (constraint_adjustment) {
+	case XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_X:
+		positioner_flip_horizontally(&flipped_positioner);
+		break;
+	case XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_Y:
+		positioner_flip_vertically(&flipped_positioner);
+		break;
+	default:
+		assert(!"Invalid constraint adjustment");
+	}
+
+	flipped_geometry = process_positioner(&flipped_positioner);
+	is_geometry_satisfied(parent_view, &flipped_geometry, output,
+			      &flipped_intersection);
+
+	if (constraint_adjustment ==
+	    XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_X) {
+		*positioner = flipped_positioner;
+		geometry->x = flipped_geometry.x;
+	} else if (constraint_adjustment ==
+		   XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_Y) {
+		*positioner = flipped_positioner;
+		geometry->y = flipped_geometry.y;
+	}
+}
+
+static void
+constrain_to_output(struct weston_desktop_surface *surface,
+		    struct weston_view *parent_view,
+		    struct weston_output *output,
+		    struct weston_geometry *geometry,
+		    struct weston_desktop_xdg_positioner *positioner)
+{
+	struct weston_geometry global_geometry;
+	int output_x2;
+	int output_y2;
+	struct weston_geometry intersection;
+
+	local_geometry_to_global(geometry, parent_view, &global_geometry);
+	output_x2 = output->pos.c.x + output->width;
+	output_y2 = output->pos.c.y + output->height;
+
+	if (is_geometry_satisfied(parent_view, geometry, output, &intersection))
+		return;
+
+	if (geometry->width != intersection.width &&
+	    (positioner->constraint_adjustment &
+	     XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_X)) {
+		try_flip_position(surface,
+				  parent_view,
+				  output,
+				  geometry,
+				  positioner,
+				  XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_X);
+	}
+	if (geometry->height != intersection.height &&
+	    (positioner->constraint_adjustment &
+	     XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_Y)) {
+		try_flip_position(surface,
+				  parent_view,
+				  output,
+				  geometry,
+				  positioner,
+				  XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_FLIP_Y);
+	}
+
+	if (is_geometry_satisfied(parent_view, geometry, output, &intersection))
+		return;
+
+	if (positioner->constraint_adjustment &
+	    XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_SLIDE_X &&
+	    global_geometry.x + global_geometry.width > output_x2) {
+		if (geometry->x < intersection.x)
+			geometry->x = intersection.x;
+		else
+			geometry->x -= geometry->width - intersection.width;
+	}
+	if (positioner->constraint_adjustment &
+	    XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_SLIDE_Y &&
+	    global_geometry.y + global_geometry.height > output_y2) {
+		if (geometry->y < intersection.y)
+			geometry->y = intersection.y;
+		else
+			geometry->y -= geometry->height - intersection.height;
+	}
+
+	if (is_geometry_satisfied(parent_view, geometry, output, &intersection))
+		return;
+
+	if (positioner->constraint_adjustment &
+	    XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_RESIZE_X) {
+		geometry->x = intersection.x;
+		geometry->width = intersection.width;
+	}
+	if (positioner->constraint_adjustment &
+	    XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_RESIZE_Y) {
+		geometry->y = intersection.y;
+		geometry->height = intersection.height;
+	}
+}
+
+static struct weston_geometry
+weston_desktop_xdg_positioner_get_geometry(struct weston_desktop_xdg_positioner *positioner,
+					   struct weston_desktop_surface *dsurface,
+					   struct weston_desktop_surface *parent)
+{
+	struct weston_geometry geometry;
+	struct weston_surface *surface;
+	struct weston_view *parent_view;
+
+	geometry = process_positioner(positioner);
+
+	if (positioner->constraint_adjustment ==
+	    XDG_POSITIONER_CONSTRAINT_ADJUSTMENT_NONE)
 		return geometry;
 
-	/* TODO: add compositor policy configuration and the code here */
+	surface = weston_desktop_surface_get_surface(parent);
+	wl_list_for_each(parent_view, &surface->views, surface_link) {
+		struct weston_output *output = parent_view->output;
+
+		constrain_to_output(dsurface, parent_view,
+				    output, &geometry, positioner);
+	}
 
 	return geometry;
 }
@@ -887,8 +1235,17 @@ weston_desktop_xdg_toplevel_resource_destroy(struct wl_resource *resource)
 	struct weston_desktop_surface *dsurface =
 		wl_resource_get_user_data(resource);
 
-	if (dsurface != NULL)
-		weston_desktop_surface_resource_destroy(resource);
+	if (dsurface) {
+		struct weston_desktop_xdg_toplevel *toplevel =
+			weston_desktop_surface_get_implementation_data(dsurface);
+		struct weston_surface *wsurface =
+			weston_desktop_surface_get_surface(dsurface);
+
+		weston_surface_unmap(wsurface);
+		wl_list_remove(wl_resource_get_link(resource));
+		toplevel->resource = NULL;
+	}
+
 }
 
 static const struct xdg_toplevel_interface weston_desktop_xdg_toplevel_implementation = {
@@ -921,8 +1278,6 @@ weston_desktop_xdg_popup_protocol_grab(struct wl_client *wl_client,
 	struct weston_seat *wseat = wl_resource_get_user_data(seat_resource);
 	struct weston_desktop_seat *seat = weston_desktop_seat_from_seat(wseat);
 	struct weston_desktop_surface *topmost;
-	bool parent_is_toplevel =
-		popup->parent->role == WESTON_DESKTOP_XDG_SURFACE_ROLE_TOPLEVEL;
 
 	/* Check that if we have a valid wseat we also got a valid desktop seat */
 	if (wseat != NULL && seat == NULL) {
@@ -930,25 +1285,15 @@ weston_desktop_xdg_popup_protocol_grab(struct wl_client *wl_client,
 		return;
 	}
 
-	if (popup->committed) {
+	if (popup->base.committed) {
 		wl_resource_post_error(popup->resource,
 				       XDG_POPUP_ERROR_INVALID_GRAB,
 				       "xdg_popup already is mapped");
 		return;
 	}
 
-	/* If seat is NULL then get_topmost_surface will return NULL. In
-	 * combination with setting parent_is_toplevel to TRUE here we will
-	 * avoid posting an error, and we will instead gracefully fail the
-	 * grab and dismiss the surface.
-	 * FIXME: this is a hack because currently we cannot check the topmost
-	 * parent with a destroyed weston_seat */
-	if (seat == NULL)
-		parent_is_toplevel = true;
-
 	topmost = weston_desktop_seat_popup_grab_get_topmost_surface(seat);
-	if ((topmost == NULL && !parent_is_toplevel) ||
-	    (topmost != NULL && topmost != popup->parent->desktop_surface)) {
+	if (topmost != NULL && topmost != popup->parent->desktop_surface) {
 		struct weston_desktop_client *client =
 			weston_desktop_surface_get_client(dsurface);
 		struct wl_resource *client_resource =
@@ -1008,8 +1353,7 @@ weston_desktop_xdg_popup_protocol_reposition(struct wl_client *wl_client,
 							   parent_dsurface);
 	popup->pending_reposition = true;
 	popup->pending_reposition_token = token;
-	if (popup->committed)
-		weston_desktop_xdg_surface_schedule_configure(&popup->base);
+	weston_desktop_xdg_surface_schedule_configure(&popup->base);
 }
 
 static void
@@ -1041,9 +1385,6 @@ weston_desktop_xdg_popup_committed(struct weston_desktop_xdg_popup *popup)
 	wl_list_for_each(view, &wsurface->views, surface_link)
 		weston_view_update_transform(view);
 
-	if (!popup->committed)
-		weston_desktop_xdg_surface_schedule_configure(&popup->base);
-	popup->committed = true;
 	weston_desktop_xdg_popup_update_position(popup->base.desktop_surface,
 						 popup);
 
@@ -1121,6 +1462,25 @@ static const struct xdg_popup_interface weston_desktop_xdg_popup_implementation
 	.reposition          = weston_desktop_xdg_popup_protocol_reposition,
 };
 
+static bool
+weston_destop_xdg_surface_resource_available(struct weston_desktop_xdg_surface *surface)
+{
+	bool resource_available = false;
+
+	switch (surface->role) {
+	case WESTON_DESKTOP_XDG_SURFACE_ROLE_NONE:
+		break;
+	case WESTON_DESKTOP_XDG_SURFACE_ROLE_TOPLEVEL:
+		resource_available = !!((struct weston_desktop_xdg_toplevel *) surface)->resource;
+		break;
+	case WESTON_DESKTOP_XDG_SURFACE_ROLE_POPUP:
+		resource_available = !!((struct weston_desktop_xdg_popup *) surface)->resource;
+		break;
+	}
+
+	return resource_available;
+}
+
 static void
 weston_desktop_xdg_surface_send_configure(void *user_data)
 {
@@ -1129,6 +1489,9 @@ weston_desktop_xdg_surface_send_configure(void *user_data)
 
 	surface->configure_idle = NULL;
 
+	if (!weston_destop_xdg_surface_resource_available(surface))
+		return;
+
 	configure = zalloc(weston_desktop_surface_configure_biggest_size);
 	if (configure == NULL) {
 		struct weston_desktop_client *client =
@@ -1213,6 +1576,9 @@ weston_desktop_xdg_surface_schedule_configure(struct weston_desktop_xdg_surface
 	struct wl_event_loop *loop = wl_display_get_event_loop(display);
 	bool pending_same = false;
 
+	if (!surface->committed)
+		return;
+
 	switch (surface->role) {
 	case WESTON_DESKTOP_XDG_SURFACE_ROLE_NONE:
 		assert(0 && "not reached");
@@ -1246,13 +1612,22 @@ weston_desktop_xdg_surface_protocol_get_toplevel(struct wl_client *wl_client,
 						 struct wl_resource *resource,
 						 uint32_t id)
 {
-	struct weston_desktop_surface *dsurface =
-		wl_resource_get_user_data(resource);
-	struct weston_surface *wsurface =
-		weston_desktop_surface_get_surface(dsurface);
-	struct weston_desktop_xdg_toplevel *toplevel =
-		weston_desktop_surface_get_implementation_data(dsurface);
-	struct weston_desktop *desktop = toplevel->base.desktop;
+	struct weston_desktop_surface *dsurface = NULL;
+	struct weston_surface *wsurface = NULL;
+	struct weston_desktop_xdg_toplevel *toplevel = NULL;
+	struct weston_desktop *desktop = NULL;
+
+	dsurface = wl_resource_get_user_data(resource);
+	if (!dsurface) {
+		wl_resource_post_error(resource,
+				XDG_SURFACE_ERROR_DEFUNCT_ROLE_OBJECT,
+				"xdg surface destroyed");
+		return;
+	}
+
+	wsurface = weston_desktop_surface_get_surface(dsurface);
+	toplevel = weston_desktop_surface_get_implementation_data(dsurface);
+	desktop = toplevel->base.desktop;
 
 	if (weston_surface_set_role(wsurface, weston_desktop_xdg_toplevel_role,
 				    resource, XDG_WM_BASE_ERROR_ROLE) < 0)
@@ -1305,18 +1680,26 @@ weston_desktop_xdg_surface_protocol_get_popup(struct wl_client *wl_client,
 					      struct wl_resource *parent_resource,
 					      struct wl_resource *positioner_resource)
 {
-	struct weston_desktop_surface *dsurface =
-		wl_resource_get_user_data(resource);
-	struct weston_surface *wsurface =
-		weston_desktop_surface_get_surface(dsurface);
-	struct weston_desktop_xdg_popup *popup =
-		weston_desktop_surface_get_implementation_data(dsurface);
+	struct weston_desktop_surface *dsurface = NULL;
+	struct weston_surface *wsurface = NULL;
+	struct weston_desktop_xdg_popup *popup = NULL;
 	struct weston_desktop_surface *parent_surface;
 	struct weston_desktop_xdg_surface *parent;
 	struct weston_desktop_xdg_positioner *positioner =
 		wl_resource_get_user_data(positioner_resource);
 	struct weston_coord_surface offset;
 
+	dsurface = wl_resource_get_user_data(resource);
+	if (!dsurface) {
+		wl_resource_post_error(resource,
+				       XDG_SURFACE_ERROR_DEFUNCT_ROLE_OBJECT,
+				       "xdg surface destroyed");
+		return;
+	}
+
+	wsurface = weston_desktop_surface_get_surface(dsurface);
+	popup = weston_desktop_surface_get_implementation_data(dsurface);
+
 	/* Popup parents are allowed to be non-null, but only if a parent is
 	 * specified 'using some other protocol' before committing. Since we
 	 * don't support such a protocol yet, clients cannot legitimately
@@ -1490,6 +1873,9 @@ weston_desktop_xdg_surface_committed(struct weston_desktop_surface *dsurface,
 	struct weston_surface *wsurface =
 		weston_desktop_surface_get_surface (dsurface);
 
+	if (!weston_destop_xdg_surface_resource_available(surface))
+		return;
+
 	if (weston_surface_has_content(wsurface) && !surface->configured) {
 		wl_resource_post_error(surface->resource,
 				       XDG_SURFACE_ERROR_UNCONFIGURED_BUFFER,
@@ -1503,6 +1889,11 @@ weston_desktop_xdg_surface_committed(struct weston_desktop_surface *dsurface,
 						    surface->next_geometry);
 	}
 
+	if (!surface->committed) {
+		surface->committed = true;
+		weston_desktop_xdg_surface_schedule_configure(surface);
+	}
+
 	switch (surface->role) {
 	case WESTON_DESKTOP_XDG_SURFACE_ROLE_NONE:
 		wl_resource_post_error(surface->resource,
