@@ -34,6 +34,7 @@
 #include "weston-debug-server-protocol.h"
 
 #include <assert.h>
+#include <inttypes.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include <string.h>
@@ -69,6 +70,7 @@ struct weston_log_context {
 	struct wl_listener compositor_destroy_listener;
 	struct wl_list scope_list; /**< weston_log_scope::compositor_link */
 	struct wl_list pending_subscription_list; /**< weston_log_subscription::source_link */
+	struct wl_list advertised_debug_list; /** weston_debug_scope_advertised::link */
 };
 
 /** weston-log message scope
@@ -118,6 +120,15 @@ struct weston_log_subscription {
 	void *data;
 };
 
+struct weston_debug_scope_advertised {
+	const char *name;
+	struct wl_list link; /** weston_log_context::advertised_debug_list */
+};
+
+static void
+weston_destroy_scopes_from_advertised_list(struct weston_log_context *ctx);
+
+
 static struct weston_log_subscription *
 find_pending_subscription(struct weston_log_context *log_ctx,
 			  const char *scope_name)
@@ -372,8 +383,10 @@ weston_debug_protocol_advertise_scopes(struct weston_log_context *log_ctx,
 				       struct wl_resource *res)
 {
 	struct weston_log_scope *scope;
+
 	wl_list_for_each(scope, &log_ctx->scope_list, compositor_link)
-		weston_debug_v1_send_available(res, scope->name, scope->desc);
+		if (weston_log_scope_to_be_advertised(log_ctx, scope->name))
+			weston_debug_v1_send_available(res, scope->name, scope->desc);
 }
 
 /** Disable debug-protocol
@@ -413,6 +426,7 @@ weston_log_ctx_create(void)
 	wl_list_init(&log_ctx->scope_list);
 	wl_list_init(&log_ctx->pending_subscription_list);
 	wl_list_init(&log_ctx->compositor_destroy_listener.link);
+	wl_list_init(&log_ctx->advertised_debug_list);
 
 	return log_ctx;
 }
@@ -449,6 +463,8 @@ weston_log_ctx_destroy(struct weston_log_context *log_ctx)
 		weston_log_subscription_destroy_pending(pending_sub);
 	}
 
+	weston_destroy_scopes_from_advertised_list(log_ctx);
+
 	/* pending_subscription_list should be empty at this point */
 
 	free(log_ctx);
@@ -522,6 +538,22 @@ weston_compositor_is_debug_protocol_enabled(struct weston_compositor *wc)
 	return wc->weston_log_ctx->global != NULL;
 }
 
+WL_EXPORT bool
+weston_log_scope_to_be_advertised(struct weston_log_context *ctx, const char *name)
+{
+	struct weston_debug_scope_advertised *entry;
+
+	if (wl_list_empty(&ctx->advertised_debug_list))
+		return true;
+
+	wl_list_for_each(entry, &ctx->advertised_debug_list, link) {
+		if (!strcmp(entry->name, name))
+			return true;
+	}
+
+	return false;
+}
+
 /** Register a new stream name, creating a log scope.
  *
  * @param log_ctx The weston_log_context where to add.
@@ -670,6 +702,33 @@ weston_compositor_add_log_scope(struct weston_compositor *compositor,
 	return scope;
 }
 
+WL_EXPORT void
+weston_add_scope_to_advertised_list(struct weston_log_context *ctx, const char *name)
+{
+	struct weston_debug_scope_advertised *advertised_scope;
+
+	if (!name || !*name)
+		return;
+
+	advertised_scope = zalloc(sizeof(*advertised_scope));
+	advertised_scope->name = strdup(name);
+
+	wl_list_insert(&ctx->advertised_debug_list, &advertised_scope->link);
+}
+
+static void
+weston_destroy_scopes_from_advertised_list(struct weston_log_context *ctx)
+{
+	struct weston_debug_scope_advertised *entry, *tmp_entry;
+
+	wl_list_for_each_safe(entry, tmp_entry, &ctx->advertised_debug_list, link) {
+		free((char *) entry->name);
+		wl_list_remove(&entry->link);
+		free(entry);
+	}
+
+}
+
 /** Destroy a log scope
  *
  * @param scope The log scope to destroy; may be NULL.
@@ -900,8 +959,8 @@ weston_log_scope_timestamp(struct weston_log_scope *scope,
 			       "%Y-%m-%d %H:%M:%S", bdt);
 
 	if (ret > 0) {
-		snprintf(buf, len, "[%s.%03ld][%s]", string,
-			 tv.tv_usec / 1000,
+		snprintf(buf, len, "[%s.%03" PRId64 "][%s]", string,
+			 (int64_t)(tv.tv_usec / 1000),
 			 (scope) ? scope->name : "no scope");
 	} else {
 		snprintf(buf, len, "[?][%s]",
@@ -954,8 +1013,8 @@ weston_log_timestamp(char *buf, size_t len, int *cached_tm_mday)
 
        strftime(timestr, sizeof(timestr), "%H:%M:%S", brokendown_time);
        /* if datestr is empty it prints only timestr*/
-       snprintf(buf, len, "%s[%s.%03li]", datestr,
-                timestr, (tv.tv_usec / 1000));
+       snprintf(buf, len, "%s[%s.%03" PRId64 "]", datestr,
+                timestr, (int64_t)(tv.tv_usec / 1000));
 
        return buf;
 }
