@@ -25,9 +25,12 @@
 
 #include "config.h"
 
+#include <libweston/linalg-3.h>
+
 #include "color-properties.h"
 #include "weston-test-client-helper.h"
 #include "weston-test-fixture-compositor.h"
+#include "weston-test-assert.h"
 #include "shared/xalloc.h"
 #include "lcms_util.h"
 
@@ -44,6 +47,7 @@ enum image_descr_info_event {
 	IMAGE_DESCR_INFO_EVENT_PRIMARIES,
 	IMAGE_DESCR_INFO_EVENT_TF_NAMED,
 	IMAGE_DESCR_INFO_EVENT_TF_POWER_EXP,
+	IMAGE_DESCR_INFO_EVENT_LUMINANCES,
 	IMAGE_DESCR_INFO_EVENT_TARGET_PRIMARIES,
 	IMAGE_DESCR_INFO_EVENT_TARGET_MAXCLL,
 	IMAGE_DESCR_INFO_EVENT_TARGET_MAXFALL,
@@ -57,15 +61,13 @@ const struct lcms_pipeline pipeline_sRGB = {
 		.Green = { 0.300, 0.600, 1.0 },
 		.Blue =  { 0.150, 0.060, 1.0 }
 	},
-	.pre_fn = TRANSFER_FN_SRGB_EOTF,
-	.mat = LCMSMAT3(1.0, 0.0, 0.0,
-			0.0, 1.0, 0.0,
-			0.0, 0.0, 1.0),
-	.post_fn = TRANSFER_FN_SRGB_EOTF_INVERSE
+	.pre_fn = TRANSFER_FN_SRGB,
+	.mat = WESTON_MAT3F_IDENTITY,
+	.post_fn = TRANSFER_FN_SRGB_INVERSE
 };
 
 struct image_description {
-	struct xx_image_description_v4 *xx_image_descr;
+	struct wp_image_description_v1 *wp_image_descr;
 
 	enum image_description_status {
 		CM_IMAGE_DESC_NOT_CREATED = 0,
@@ -81,10 +83,11 @@ struct image_description {
 	uint32_t icc_size;
 
 	/* For parametric images descriptions. */
-	enum xx_color_manager_v4_primaries primaries_named;
+	enum wp_color_manager_v1_primaries primaries_named;
 	struct weston_color_gamut primaries;
-	enum xx_color_manager_v4_transfer_function tf_named;
+	enum wp_color_manager_v1_transfer_function tf_named;
 	float tf_power;
+	float min_lum, max_lum, ref_lum;
 	struct weston_color_gamut target_primaries;
 	float target_min_lum, target_max_lum;
 	float target_max_cll;
@@ -92,7 +95,7 @@ struct image_description {
 };
 
 struct image_description_info {
-	struct xx_image_description_info_v4 *xx_image_description_info;
+	struct wp_image_description_info_v1 *wp_image_description_info;
 	struct image_description *image_descr;
 
 	/* Bitfield that holds what events the compositor has sent us through
@@ -102,11 +105,11 @@ struct image_description_info {
 };
 
 struct color_manager {
-        struct xx_color_manager_v4 *manager;
+        struct wp_color_manager_v1 *manager;
 
-	struct xx_color_management_output_v4 *output;
-	struct xx_color_management_surface_v4 *surface;
-	struct xx_color_management_feedback_surface_v4 *feedback_surface;
+	struct wp_color_management_output_v1 *output;
+	struct wp_color_management_surface_v1 *surface;
+	struct wp_color_management_surface_feedback_v1 *surface_feedback;
 
 	struct wl_list image_descr_list; /* image_description::link */
 
@@ -117,6 +120,8 @@ struct color_manager {
 	/* Bitfield that holds what rendering intents are supported. If enum
 	 * supported_render_intent v is supported, bit v will be set to 1. */
 	uint32_t supported_rendering_intents;
+
+	bool done;
 };
 
 static struct image_description *
@@ -131,12 +136,12 @@ static void
 image_description_destroy(struct image_description *image_descr)
 {
 	wl_list_remove(&image_descr->link);
-	xx_image_description_v4_destroy(image_descr->xx_image_descr);
+	wp_image_description_v1_destroy(image_descr->wp_image_descr);
 	free(image_descr);
 }
 
 static void
-image_descr_ready(void *data, struct xx_image_description_v4 *xx_image_description_v4,
+image_descr_ready(void *data, struct wp_image_description_v1 *wp_image_description_v1,
 		  uint32_t identity)
 {
 	struct image_description *image_descr = data;
@@ -145,7 +150,7 @@ image_descr_ready(void *data, struct xx_image_description_v4 *xx_image_descripti
 }
 
 static void
-image_descr_failed(void *data, struct xx_image_description_v4 *xx_image_description_v4,
+image_descr_failed(void *data, struct wp_image_description_v1 *wp_image_description_v1,
 		   uint32_t cause, const char *msg)
 {
 	struct image_description *image_descr = data;
@@ -156,7 +161,7 @@ image_descr_failed(void *data, struct xx_image_description_v4 *xx_image_descript
 		"    cause: %u, msg: %s\n", cause, msg);
 }
 
-static const struct xx_image_description_v4_listener
+static const struct wp_image_description_v1_listener
 image_descr_iface = {
 	.ready = image_descr_ready,
 	.failed = image_descr_failed,
@@ -166,15 +171,13 @@ static void
 image_descr_info_received(struct image_description_info *image_descr_info,
 			  enum image_descr_info_event ev)
 {
-	/* TODO: replace this assert with weston_assert_uint32_mask_bit_is_clear
-	 * when we start using weston-assert in the test suite. */
-	assert(!((image_descr_info->events_received >> ev) & 1));
+	test_assert_bit_not_set(image_descr_info->events_received, 1 << ev);
 	image_descr_info->events_received |= (1 << ev);
 }
 
 static void
 image_descr_info_primaries(void *data,
-			   struct xx_image_description_info_v4 *xx_image_description_info_v4,
+			   struct wp_image_description_info_v1 *wp_image_description_info_v1,
 			   int32_t r_x, int32_t r_y, int32_t g_x, int32_t g_y,
 			   int32_t b_x, int32_t b_y, int32_t w_x, int32_t w_y)
 {
@@ -185,19 +188,19 @@ image_descr_info_primaries(void *data,
 	image_descr_info_received(image_descr_info,
 				  IMAGE_DESCR_INFO_EVENT_PRIMARIES);
 
-	image_descr->primaries.primary[0].x = r_x / 10000.0;
-	image_descr->primaries.primary[0].y = r_y / 10000.0;
-	image_descr->primaries.primary[1].x = g_x / 10000.0;
-	image_descr->primaries.primary[1].y = g_y / 10000.0;
-	image_descr->primaries.primary[2].x = b_x / 10000.0;
-	image_descr->primaries.primary[2].y = b_y / 10000.0;
-	image_descr->primaries.white_point.x = w_x / 10000.0;
-	image_descr->primaries.white_point.y = w_y / 10000.0;
+	image_descr->primaries.primary[0].x = r_x / 1000000.0f;
+	image_descr->primaries.primary[0].y = r_y / 1000000.0f;
+	image_descr->primaries.primary[1].x = g_x / 1000000.0f;
+	image_descr->primaries.primary[1].y = g_y / 1000000.0f;
+	image_descr->primaries.primary[2].x = b_x / 1000000.0f;
+	image_descr->primaries.primary[2].y = b_y / 1000000.0f;
+	image_descr->primaries.white_point.x = w_x / 1000000.0f;
+	image_descr->primaries.white_point.y = w_y / 1000000.0f;
 }
 
 static void
 image_descr_info_primaries_named(void *data,
-				 struct xx_image_description_info_v4 *xx_image_description_info_v4,
+				 struct wp_image_description_info_v1 *wp_image_description_info_v1,
 				 uint32_t primaries)
 {
 	struct image_description_info *image_descr_info = data;
@@ -211,7 +214,7 @@ image_descr_info_primaries_named(void *data,
 
 static void
 image_descr_info_tf_named(void *data,
-			  struct xx_image_description_info_v4 *xx_image_description_info_v4,
+			  struct wp_image_description_info_v1 *wp_image_description_info_v1,
 			  uint32_t tf)
 {
 	struct image_description_info *image_descr_info = data;
@@ -225,7 +228,7 @@ image_descr_info_tf_named(void *data,
 
 static void
 image_descr_info_tf_power(void *data,
-			  struct xx_image_description_info_v4 *xx_image_description_info_v4,
+			  struct wp_image_description_info_v1 *wp_image_description_info_v1,
 			  uint32_t tf_power)
 {
 	struct image_description_info *image_descr_info = data;
@@ -237,9 +240,25 @@ image_descr_info_tf_power(void *data,
 	image_descr->tf_power = tf_power / 10000.0;
 }
 
+static void
+image_descr_info_luminances(void *data,
+			    struct wp_image_description_info_v1 *wp_image_description_info_v1,
+			    uint32_t min_lum, uint32_t max_lum, uint32_t ref_lum)
+{
+	struct image_description_info *image_descr_info = data;
+	struct image_description *image_descr = image_descr_info->image_descr;
+
+	image_descr_info_received(image_descr_info,
+				  IMAGE_DESCR_INFO_EVENT_LUMINANCES);
+
+	image_descr->min_lum = min_lum / 10000.0;
+	image_descr->max_lum = max_lum;
+	image_descr->ref_lum = ref_lum;
+}
+
 static void
 image_descr_info_target_primaries(void *data,
-				  struct xx_image_description_info_v4 *xx_image_description_info_v4,
+				  struct wp_image_description_info_v1 *wp_image_description_info_v1,
 				  int32_t r_x, int32_t r_y, int32_t g_x, int32_t g_y,
 				  int32_t b_x, int32_t b_y, int32_t w_x, int32_t w_y)
 {
@@ -249,19 +268,19 @@ image_descr_info_target_primaries(void *data,
 	image_descr_info_received(image_descr_info,
 				  IMAGE_DESCR_INFO_EVENT_TARGET_PRIMARIES);
 
-	image_descr->target_primaries.primary[0].x = r_x / 10000.0;
-	image_descr->target_primaries.primary[0].y = r_y / 10000.0;
-	image_descr->target_primaries.primary[1].x = g_x / 10000.0;
-	image_descr->target_primaries.primary[1].y = g_y / 10000.0;
-	image_descr->target_primaries.primary[2].x = b_x / 10000.0;
-	image_descr->target_primaries.primary[2].y = b_y / 10000.0;
-	image_descr->target_primaries.white_point.x = w_x / 10000.0;
-	image_descr->target_primaries.white_point.y = w_y / 10000.0;
+	image_descr->target_primaries.primary[0].x = r_x / 1000000.0f;
+	image_descr->target_primaries.primary[0].y = r_y / 1000000.0f;
+	image_descr->target_primaries.primary[1].x = g_x / 1000000.0f;
+	image_descr->target_primaries.primary[1].y = g_y / 1000000.0f;
+	image_descr->target_primaries.primary[2].x = b_x / 1000000.0f;
+	image_descr->target_primaries.primary[2].y = b_y / 1000000.0f;
+	image_descr->target_primaries.white_point.x = w_x / 1000000.0f;
+	image_descr->target_primaries.white_point.y = w_y / 1000000.0f;
 }
 
 static void
 image_descr_info_target_luminance(void *data,
-				  struct xx_image_description_info_v4 *xx_image_description_info_v4,
+				  struct wp_image_description_info_v1 *wp_image_description_info_v1,
 				  uint32_t min_lum, uint32_t max_lum)
 {
 	struct image_description_info *image_descr_info = data;
@@ -276,7 +295,7 @@ image_descr_info_target_luminance(void *data,
 
 static void
 image_descr_info_target_max_cll(void *data,
-				struct xx_image_description_info_v4 *xx_image_description_info_v4,
+				struct wp_image_description_info_v1 *wp_image_description_info_v1,
 				uint32_t maxCLL)
 {
 	struct image_description_info *image_descr_info = data;
@@ -290,7 +309,7 @@ image_descr_info_target_max_cll(void *data,
 
 static void
 image_descr_info_target_max_fall(void *data,
-				 struct xx_image_description_info_v4 *xx_image_description_info_v4,
+				 struct wp_image_description_info_v1 *wp_image_description_info_v1,
 				 uint32_t maxFALL)
 {
 	struct image_description_info *image_descr_info = data;
@@ -304,7 +323,7 @@ image_descr_info_target_max_fall(void *data,
 
 static void
 image_descr_info_icc_file_event(void *data,
-				struct xx_image_description_info_v4 *xx_image_description_info_v4,
+				struct wp_image_description_info_v1 *wp_image_description_info_v1,
 				int32_t icc_fd, uint32_t icc_size)
 {
 	struct image_description_info *image_descr_info = data;
@@ -337,6 +356,8 @@ are_events_received_valid(struct image_description_info *image_descr_info)
 	 * parameters make sense. */
 	bool received_primaries, received_primaries_named;
 	bool received_tf_power, received_tf_named;
+	bool received_target_primaries;
+	bool received_lum, received_target_lum;
 
 	/* Should have received the primaries somewhow. */
 	received_primaries_named = (events_received >>
@@ -362,19 +383,19 @@ are_events_received_valid(struct image_description_info *image_descr_info)
 
 	/* If we received tf named and exp power, they must match. */
 	if (received_tf_named && received_tf_power) {
-		if (image_descr->tf_named != XX_COLOR_MANAGER_V4_TRANSFER_FUNCTION_GAMMA22 &&
-		    image_descr->tf_named != XX_COLOR_MANAGER_V4_TRANSFER_FUNCTION_GAMMA28) {
+		if (image_descr->tf_named != WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_GAMMA22 &&
+		    image_descr->tf_named != WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_GAMMA28) {
 			testlog("    Error: parametric image description tf " \
 				"named is not pure power-law, but still received " \
 				"tf power event\n");
 			return false;
-		} else if (image_descr->tf_named == XX_COLOR_MANAGER_V4_TRANSFER_FUNCTION_GAMMA22 &&
+		} else if (image_descr->tf_named == WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_GAMMA22 &&
 			   image_descr->tf_power != 2.2f) {
 			testlog("    Error: parametric image description tf named " \
 				"is pure power-law 2.2, but tf power received is %f\n",
 				image_descr->tf_power);
 			return false;
-		} else if (image_descr->tf_named == XX_COLOR_MANAGER_V4_TRANSFER_FUNCTION_GAMMA28 &&
+		} else if (image_descr->tf_named == WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_GAMMA28 &&
 			   image_descr->tf_power != 2.8f) {
 			testlog("    Error: parametric image description tf named " \
 				"is pure power-law 2.8, but tf power received is %f\n",
@@ -383,22 +404,46 @@ are_events_received_valid(struct image_description_info *image_descr_info)
 		}
 	}
 
-	/* TODO: when target primaries, luminance, maxcll and maxfall are
-	 * allowed? */
+	/* We should receive luminance. */
+	received_lum = (events_received >>
+			IMAGE_DESCR_INFO_EVENT_LUMINANCES) & 1;
+	if (!received_lum) {
+		testlog("    Error: parametric image description but no " \
+			"luminances received\n");
+		return false;
+	}
+
+	/* We should receive target primaries. */
+	received_target_primaries = (events_received >>
+				     IMAGE_DESCR_INFO_EVENT_TARGET_PRIMARIES) & 1;
+	if (!received_target_primaries) {
+		testlog("    Error: parametric image description but no " \
+			"target primaries received\n");
+		return false;
+	}
+
+	/* We should receive target luminance. */
+	received_target_lum = (events_received >>
+			       IMAGE_DESCR_INFO_EVENT_TARGET_LUMINANCE) & 1;
+	if (!received_target_lum) {
+		testlog("    Error: parametric image description but no " \
+			"target luminances received\n");
+		return false;
+	}
 
 	return true;
 }
 
 static void
 image_descr_info_done(void *data,
-		      struct xx_image_description_info_v4 *xx_image_description_info_v4)
+		      struct wp_image_description_info_v1 *wp_image_description_info_v1)
 {
 	struct image_description_info *image_descr_info = data;
 	struct image_description *image_descr = image_descr_info->image_descr;
 
-	testlog("Image description info %p done:\n", xx_image_description_info_v4);
+	testlog("Image description info %p done:\n", wp_image_description_info_v1);
 
-	assert(are_events_received_valid(image_descr_info));
+	test_assert_true(are_events_received_valid(image_descr_info));
 
 	/* ICC based image description */
 	if ((image_descr_info->events_received >> IMAGE_DESCR_INFO_EVENT_ICC_FD) & 1) {
@@ -459,12 +504,13 @@ image_descr_info_done(void *data,
 		testlog("    Target maxFALL: %.4f\n", image_descr->target_max_fall);
 }
 
-static const struct xx_image_description_info_v4_listener
+static const struct wp_image_description_info_v1_listener
 image_descr_info_iface = {
 	.primaries = image_descr_info_primaries,
 	.primaries_named = image_descr_info_primaries_named,
 	.tf_named = image_descr_info_tf_named,
 	.tf_power = image_descr_info_tf_power,
+	.luminances = image_descr_info_luminances,
 	.target_primaries = image_descr_info_target_primaries,
 	.target_luminance = image_descr_info_target_luminance,
 	.target_max_cll = image_descr_info_target_max_cll,
@@ -474,7 +520,7 @@ image_descr_info_iface = {
 };
 
 static void
-cm_supported_intent(void *data, struct xx_color_manager_v4 *xx_color_manager_v4,
+cm_supported_intent(void *data, struct wp_color_manager_v1 *wp_color_manager_v1,
 		    uint32_t render_intent)
 {
 	struct color_manager *cm = data;
@@ -483,7 +529,7 @@ cm_supported_intent(void *data, struct xx_color_manager_v4 *xx_color_manager_v4,
 }
 
 static void
-cm_supported_feature(void *data, struct xx_color_manager_v4 *xx_color_manager_v4,
+cm_supported_feature(void *data, struct wp_color_manager_v1 *wp_color_manager_v1,
 		     uint32_t feature)
 {
 	struct color_manager *cm = data;
@@ -492,27 +538,36 @@ cm_supported_feature(void *data, struct xx_color_manager_v4 *xx_color_manager_v4
 }
 
 static void
-cm_supported_tf_named(void *data, struct xx_color_manager_v4 *xx_color_manager_v4,
+cm_supported_tf_named(void *data, struct wp_color_manager_v1 *wp_color_manager_v1,
 		      uint32_t tf_code)
 {
-	/* only used to create image descriptions using parameters, which is
-	 * still unsupported by Weston. */
+	/* only used to create image descriptions using parameters, which we
+	 * won't do in this test file. */
 }
 
 static void
-cm_supported_primaries_named(void *data, struct xx_color_manager_v4 *xx_color_manager_v4,
+cm_supported_primaries_named(void *data, struct wp_color_manager_v1 *wp_color_manager_v1,
 			     uint32_t primaries_code)
 {
-	/* only used to create image descriptions using parameters, which is
-	 * still unsupported by Weston. */
+	/* only used to create image descriptions using parameters, which we
+	 * won't do in this test file. */
 }
 
-static const struct xx_color_manager_v4_listener
+static void
+cm_done(void *data, struct wp_color_manager_v1 *wp_color_manager_v1)
+{
+	struct color_manager *cm = data;
+
+	cm->done = true;
+}
+
+static const struct wp_color_manager_v1_listener
 cm_iface = {
 	.supported_intent = cm_supported_intent,
 	.supported_feature = cm_supported_feature,
 	.supported_tf_named = cm_supported_tf_named,
 	.supported_primaries_named = cm_supported_primaries_named,
+	.done = cm_done,
 };
 
 static void
@@ -523,32 +578,41 @@ color_manager_init(struct color_manager *cm, struct client *client)
 	wl_list_init(&cm->image_descr_list);
 
         cm->manager = bind_to_singleton_global(client,
-					       &xx_color_manager_v4_interface,
+					       &wp_color_manager_v1_interface,
 					       1);
-	xx_color_manager_v4_add_listener(cm->manager, &cm_iface, cm);
+	wp_color_manager_v1_add_listener(cm->manager, &cm_iface, cm);
 
-	cm->output = xx_color_manager_v4_get_output(cm->manager,
+	cm->output = wp_color_manager_v1_get_output(cm->manager,
 						    client->output->wl_output);
 
-	cm->surface = xx_color_manager_v4_get_surface(cm->manager,
+	cm->surface = wp_color_manager_v1_get_surface(cm->manager,
 						      client->surface->wl_surface);
 
-	cm->feedback_surface =
-		xx_color_manager_v4_get_feedback_surface(cm->manager,
+	cm->surface_feedback =
+		wp_color_manager_v1_get_surface_feedback(cm->manager,
 							 client->surface->wl_surface);
 
 	client_roundtrip(client);
 
-	/* For now, Weston only supports the ICC image description creator. All
-	 * the parametric parts of the protocol are still unsupported. */
-	assert(cm->supported_features == (1 << XX_COLOR_MANAGER_V4_FEATURE_ICC_V2_V4));
+	/* Weston supports all color features. */
+	test_assert_u32_eq(cm->supported_features,
+			   (1 << WP_COLOR_MANAGER_V1_FEATURE_ICC_V2_V4) |
+			   (1 << WP_COLOR_MANAGER_V1_FEATURE_PARAMETRIC) |
+			   (1 << WP_COLOR_MANAGER_V1_FEATURE_SET_PRIMARIES) |
+			   (1 << WP_COLOR_MANAGER_V1_FEATURE_SET_TF_POWER) |
+			   (1 << WP_COLOR_MANAGER_V1_FEATURE_SET_LUMINANCES) |
+			   (1 << WP_COLOR_MANAGER_V1_FEATURE_SET_MASTERING_DISPLAY_PRIMARIES) |
+			   (1 << WP_COLOR_MANAGER_V1_FEATURE_EXTENDED_TARGET_VOLUME));
 
 	/* Weston supports all rendering intents. */
-	assert(cm->supported_rendering_intents == ((1 << XX_COLOR_MANAGER_V4_RENDER_INTENT_PERCEPTUAL) |
-						   (1 << XX_COLOR_MANAGER_V4_RENDER_INTENT_RELATIVE) |
-						   (1 << XX_COLOR_MANAGER_V4_RENDER_INTENT_SATURATION) |
-						   (1 << XX_COLOR_MANAGER_V4_RENDER_INTENT_ABSOLUTE) |
-						   (1 << XX_COLOR_MANAGER_V4_RENDER_INTENT_RELATIVE_BPC)));
+	test_assert_u32_eq(cm->supported_rendering_intents,
+			   (1 << WP_COLOR_MANAGER_V1_RENDER_INTENT_PERCEPTUAL) |
+			   (1 << WP_COLOR_MANAGER_V1_RENDER_INTENT_RELATIVE) |
+			   (1 << WP_COLOR_MANAGER_V1_RENDER_INTENT_SATURATION) |
+			   (1 << WP_COLOR_MANAGER_V1_RENDER_INTENT_ABSOLUTE) |
+			   (1 << WP_COLOR_MANAGER_V1_RENDER_INTENT_RELATIVE_BPC));
+
+	test_assert_true(cm->done);
 }
 
 static void
@@ -559,10 +623,10 @@ color_manager_fini(struct color_manager *cm)
 	wl_list_for_each_safe(image_descr, tmp, &cm->image_descr_list, link)
 		image_description_destroy(image_descr);
 
-	xx_color_management_output_v4_destroy(cm->output);
-	xx_color_management_surface_v4_destroy(cm->surface);
-	xx_color_management_feedback_surface_v4_destroy(cm->feedback_surface);
-	xx_color_manager_v4_destroy(cm->manager);
+	wp_color_management_output_v1_destroy(cm->output);
+	wp_color_management_surface_v1_destroy(cm->surface);
+	wp_color_management_surface_feedback_v1_destroy(cm->surface_feedback);
+	wp_color_manager_v1_destroy(cm->manager);
 }
 
 static struct image_description *
@@ -570,10 +634,10 @@ get_output_image_description(struct color_manager *cm)
 {
 	struct image_description *image_descr = image_description_create();
 
-	image_descr->xx_image_descr =
-		xx_color_management_output_v4_get_image_description(cm->output);
+	image_descr->wp_image_descr =
+		wp_color_management_output_v1_get_image_description(cm->output);
 
-	xx_image_description_v4_add_listener(image_descr->xx_image_descr,
+	wp_image_description_v1_add_listener(image_descr->wp_image_descr,
 					     &image_descr_iface, image_descr);
 
 	wl_list_insert(&cm->image_descr_list, &image_descr->link);
@@ -586,10 +650,10 @@ get_surface_preferred_image_description(struct color_manager *cm)
 {
 	struct image_description *image_descr = image_description_create();
 
-	image_descr->xx_image_descr =
-		xx_color_management_feedback_surface_v4_get_preferred(cm->feedback_surface);
+	image_descr->wp_image_descr =
+		wp_color_management_surface_feedback_v1_get_preferred(cm->surface_feedback);
 
-	xx_image_description_v4_add_listener(image_descr->xx_image_descr,
+	wp_image_description_v1_add_listener(image_descr->wp_image_descr,
 					     &image_descr_iface, image_descr);
 
 	wl_list_insert(&cm->image_descr_list, &image_descr->link);
@@ -599,7 +663,7 @@ get_surface_preferred_image_description(struct color_manager *cm)
 
 static struct image_description *
 create_icc_based_image_description(struct color_manager *cm,
-				   struct xx_image_description_creator_icc_v4 *image_descr_creator_icc,
+				   struct wp_image_description_creator_icc_v1 *image_descr_creator_icc,
 				   const char *icc_path)
 {
 	struct image_description *image_descr = image_description_create();
@@ -607,16 +671,16 @@ create_icc_based_image_description(struct color_manager *cm,
 	struct stat st;
 
 	icc_fd = open(icc_path, O_RDONLY);
-	assert(icc_fd >= 0);
+	test_assert_s32_ge(icc_fd, 0);
 
-	assert(fstat(icc_fd, &st) == 0);
+	test_assert_int_eq(fstat(icc_fd, &st), 0);
 
-	xx_image_description_creator_icc_v4_set_icc_file(image_descr_creator_icc,
+	wp_image_description_creator_icc_v1_set_icc_file(image_descr_creator_icc,
 							 icc_fd, 0, st.st_size);
-	image_descr->xx_image_descr =
-		xx_image_description_creator_icc_v4_create(image_descr_creator_icc);
+	image_descr->wp_image_descr =
+		wp_image_description_creator_icc_v1_create(image_descr_creator_icc);
 
-	xx_image_description_v4_add_listener(image_descr->xx_image_descr,
+	wp_image_description_v1_add_listener(image_descr->wp_image_descr,
 					     &image_descr_iface, image_descr);
 
 	wl_list_insert(&cm->image_descr_list, &image_descr->link);
@@ -635,10 +699,10 @@ build_sRGB_icc_profile(const char *filename)
 
 	profile = build_lcms_matrix_shaper_profile_output(NULL, &pipeline_sRGB,
 							  vcgt_exponents);
-	assert(profile);
+	test_assert_ptr_not_null(profile);
 
 	saved = cmsSaveProfileToFile(profile, filename);
-	assert(saved);
+	test_assert_true(saved);
 
 	cmsCloseProfile(profile);
 }
@@ -660,7 +724,7 @@ fixture_setup(struct weston_test_harness *harness)
 
 		tmp = output_filename_for_test_program(THIS_TEST_NAME,
 						       NULL, "icm");
-		assert(strlen(tmp) < ARRAY_LENGTH(srgb_icc_profile_path));
+		test_assert_int_lt(strlen(tmp), ARRAY_LENGTH(srgb_icc_profile_path));
 		strcpy(srgb_icc_profile_path, tmp);
 		free(tmp);
 
@@ -685,12 +749,14 @@ TEST(smoke_test)
 
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 static void
 image_descr_info_destroy(struct image_description_info *image_descr_info)
 {
-	xx_image_description_info_v4_destroy(image_descr_info->xx_image_description_info);
+	wp_image_description_info_v1_destroy(image_descr_info->wp_image_description_info);
 	free(image_descr_info);
 }
 
@@ -703,10 +769,10 @@ image_descr_get_information(struct image_description *image_descr)
 
 	image_descr_info->image_descr = image_descr;
 
-	image_descr_info->xx_image_description_info =
-		xx_image_description_v4_get_information(image_descr->xx_image_descr);
+	image_descr_info->wp_image_description_info =
+		wp_image_description_v1_get_information(image_descr->wp_image_descr);
 
-	xx_image_description_info_v4_add_listener(image_descr_info->xx_image_description_info,
+	wp_image_description_info_v1_add_listener(image_descr_info->wp_image_description_info,
 						  &image_descr_info_iface,
 						  image_descr_info);
 
@@ -718,9 +784,10 @@ wait_until_image_description_ready(struct client *client,
 				   struct image_description *image_descr)
 {
 	while (image_descr->status == CM_IMAGE_DESC_NOT_CREATED)
-		assert(wl_display_dispatch(client->wl_display) >= 0);
+		if (!test_assert_int_ge(wl_display_dispatch(client->wl_display), 0))
+			break;
 
-	assert(image_descr->status == CM_IMAGE_DESC_READY);
+	test_assert_enum(image_descr->status, CM_IMAGE_DESC_READY);
 }
 
 TEST(output_get_image_description)
@@ -744,6 +811,8 @@ TEST(output_get_image_description)
 	image_descr_info_destroy(image_descr_info);
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(surface_get_preferred_image_description)
@@ -767,39 +836,22 @@ TEST(surface_get_preferred_image_description)
 	image_descr_info_destroy(image_descr_info);
 	color_manager_fini(&cm);
 	client_destroy(client);
-}
 
-TEST(create_parametric_image_description_creator_object)
-{
-	struct client *client;
-	struct color_manager cm;
-	struct xx_image_description_creator_params_v4 *param_creator;
-
-	client = create_client_and_test_surface(100, 100, 100, 100);
-	color_manager_init(&cm, client);
-
-	/* Parametric image description creator is still unsupported */
-	param_creator = xx_color_manager_v4_new_parametric_creator(cm.manager);
-	expect_protocol_error(client, &xx_color_manager_v4_interface,
-			      XX_COLOR_MANAGER_V4_ERROR_UNSUPPORTED_FEATURE);
-
-	xx_image_description_creator_params_v4_destroy(param_creator);
-	color_manager_fini(&cm);
-	client_destroy(client);
+	return RESULT_OK;
 }
 
 TEST(create_image_description_before_setting_icc_file)
 {
 	struct client *client;
 	struct color_manager cm;
-	struct xx_image_description_creator_icc_v4 *image_descr_creator_icc;
-	struct xx_image_description_v4 *image_desc;
+	struct wp_image_description_creator_icc_v1 *image_descr_creator_icc;
+	struct wp_image_description_v1 *image_desc;
 
 	client = create_client_and_test_surface(100, 100, 100, 100);
 	color_manager_init(&cm, client);
 
 	image_descr_creator_icc =
-		xx_color_manager_v4_new_icc_creator(cm.manager);
+		wp_color_manager_v1_create_icc_creator(cm.manager);
 
 	/* Try creating image description based on ICC profile but without
 	 * setting the ICC file, what should fail.
@@ -807,20 +859,22 @@ TEST(create_image_description_before_setting_icc_file)
 	 * We expect a protocol error from unknown object, because the
 	 * image_descr_creator_icc wl_proxy will get destroyed with the create
 	 * call below. It is a destructor request. */
-	image_desc = xx_image_description_creator_icc_v4_create(image_descr_creator_icc);
+	image_desc = wp_image_description_creator_icc_v1_create(image_descr_creator_icc);
 	expect_protocol_error(client, NULL,
-			      XX_IMAGE_DESCRIPTION_CREATOR_ICC_V4_ERROR_INCOMPLETE_SET);
+			      WP_IMAGE_DESCRIPTION_CREATOR_ICC_V1_ERROR_INCOMPLETE_SET);
 
-	xx_image_description_v4_destroy(image_desc);
+	wp_image_description_v1_destroy(image_desc);
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(set_unreadable_icc_fd)
 {
 	struct client *client;
 	struct color_manager cm;
-	struct xx_image_description_creator_icc_v4 *image_descr_creator_icc;
+	struct wp_image_description_creator_icc_v1 *image_descr_creator_icc;
 	int32_t icc_fd;
 	struct stat st;
 
@@ -828,89 +882,95 @@ TEST(set_unreadable_icc_fd)
 	color_manager_init(&cm, client);
 
 	image_descr_creator_icc =
-		xx_color_manager_v4_new_icc_creator(cm.manager);
+		wp_color_manager_v1_create_icc_creator(cm.manager);
 
 	/* The file is being open with WRITE, not READ permission. So the
 	 * compositor should complain. */
 	icc_fd = open(srgb_icc_profile_path, O_WRONLY);
-	assert(icc_fd >= 0);
-	assert(fstat(icc_fd, &st) == 0);
+	test_assert_s32_ge(icc_fd, 0);
+	test_assert_int_eq(fstat(icc_fd, &st), 0);
 
 	/* Try setting the bad ICC file fd, it should fail. */
-	xx_image_description_creator_icc_v4_set_icc_file(image_descr_creator_icc,
+	wp_image_description_creator_icc_v1_set_icc_file(image_descr_creator_icc,
 							 icc_fd, 0, st.st_size);
-	expect_protocol_error(client, &xx_image_description_creator_icc_v4_interface,
-			      XX_IMAGE_DESCRIPTION_CREATOR_ICC_V4_ERROR_BAD_FD);
+	expect_protocol_error(client, &wp_image_description_creator_icc_v1_interface,
+			      WP_IMAGE_DESCRIPTION_CREATOR_ICC_V1_ERROR_BAD_FD);
 
 	close(icc_fd);
-	xx_image_description_creator_icc_v4_destroy(image_descr_creator_icc);
+	wp_image_description_creator_icc_v1_destroy(image_descr_creator_icc);
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(set_bad_icc_size_zero)
 {
 	struct client *client;
 	struct color_manager cm;
-	struct xx_image_description_creator_icc_v4 *image_descr_creator_icc;
+	struct wp_image_description_creator_icc_v1 *image_descr_creator_icc;
 	int32_t icc_fd;
 
 	client = create_client_and_test_surface(100, 100, 100, 100);
 	color_manager_init(&cm, client);
 
 	image_descr_creator_icc =
-		xx_color_manager_v4_new_icc_creator(cm.manager);
+		wp_color_manager_v1_create_icc_creator(cm.manager);
 
 	icc_fd = open(srgb_icc_profile_path, O_RDONLY);
-	assert(icc_fd >= 0);
+	test_assert_s32_ge(icc_fd, 0);
 
 	/* Try setting ICC file with a bad size, it should fail. */
-	xx_image_description_creator_icc_v4_set_icc_file(image_descr_creator_icc,
+	wp_image_description_creator_icc_v1_set_icc_file(image_descr_creator_icc,
 							 icc_fd, 0, 0);
-	expect_protocol_error(client, &xx_image_description_creator_icc_v4_interface,
-			      XX_IMAGE_DESCRIPTION_CREATOR_ICC_V4_ERROR_BAD_SIZE);
+	expect_protocol_error(client, &wp_image_description_creator_icc_v1_interface,
+			      WP_IMAGE_DESCRIPTION_CREATOR_ICC_V1_ERROR_BAD_SIZE);
 
 	close(icc_fd);
-	xx_image_description_creator_icc_v4_destroy(image_descr_creator_icc);
+	wp_image_description_creator_icc_v1_destroy(image_descr_creator_icc);
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(set_bad_icc_non_seekable)
 {
 	struct client *client;
 	struct color_manager cm;
-	struct xx_image_description_creator_icc_v4 *image_descr_creator_icc;
+	struct wp_image_description_creator_icc_v1 *image_descr_creator_icc;
 	int32_t fds[2];
 
 	client = create_client_and_test_surface(100, 100, 100, 100);
 	color_manager_init(&cm, client);
 
 	image_descr_creator_icc =
-		xx_color_manager_v4_new_icc_creator(cm.manager);
+		wp_color_manager_v1_create_icc_creator(cm.manager);
 
 	/* We need a non-seekable file, and pipes are non-seekable. */
-	assert(pipe(fds) >= 0);
+	test_assert_int_ge(pipe(fds), 0);
 
 	/* Pretend that it has a valid size of 1024 bytes. That still should
 	 * fail because the fd is non-seekable. */
-	xx_image_description_creator_icc_v4_set_icc_file(image_descr_creator_icc,
+	wp_image_description_creator_icc_v1_set_icc_file(image_descr_creator_icc,
 							 fds[0], 0, 1024);
-	expect_protocol_error(client, &xx_image_description_creator_icc_v4_interface,
-			      XX_IMAGE_DESCRIPTION_CREATOR_ICC_V4_ERROR_BAD_FD);
+	expect_protocol_error(client, &wp_image_description_creator_icc_v1_interface,
+			      WP_IMAGE_DESCRIPTION_CREATOR_ICC_V1_ERROR_BAD_FD);
 
 	close(fds[0]);
 	close(fds[1]);
-	xx_image_description_creator_icc_v4_destroy(image_descr_creator_icc);
+	wp_image_description_creator_icc_v1_destroy(image_descr_creator_icc);
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(set_icc_twice)
 {
 	struct client *client;
 	struct color_manager cm;
-	struct xx_image_description_creator_icc_v4 *image_descr_creator_icc;
+	struct wp_image_description_creator_icc_v1 *image_descr_creator_icc;
 	int32_t icc_fd;
 	struct stat st;
 
@@ -918,33 +978,35 @@ TEST(set_icc_twice)
 	color_manager_init(&cm, client);
 
 	image_descr_creator_icc =
-		xx_color_manager_v4_new_icc_creator(cm.manager);
+		wp_color_manager_v1_create_icc_creator(cm.manager);
 
 	icc_fd = open(srgb_icc_profile_path, O_RDONLY);
-	assert(icc_fd >= 0);
-	assert(fstat(icc_fd, &st) == 0);
+	test_assert_s32_ge(icc_fd, 0);
+	test_assert_int_eq(fstat(icc_fd, &st), 0);
 
-	xx_image_description_creator_icc_v4_set_icc_file(image_descr_creator_icc,
+	wp_image_description_creator_icc_v1_set_icc_file(image_descr_creator_icc,
 							 icc_fd, 0, st.st_size);
 	client_roundtrip(client);
 
 	/* Set the ICC again, what should fail. */
-	xx_image_description_creator_icc_v4_set_icc_file(image_descr_creator_icc,
+	wp_image_description_creator_icc_v1_set_icc_file(image_descr_creator_icc,
 							 icc_fd, 0, st.st_size);
-	expect_protocol_error(client, &xx_image_description_creator_icc_v4_interface,
-			      XX_IMAGE_DESCRIPTION_CREATOR_ICC_V4_ERROR_ALREADY_SET);
+	expect_protocol_error(client, &wp_image_description_creator_icc_v1_interface,
+			      WP_IMAGE_DESCRIPTION_CREATOR_ICC_V1_ERROR_ALREADY_SET);
 
 	close(icc_fd);
-	xx_image_description_creator_icc_v4_destroy(image_descr_creator_icc);
+	wp_image_description_creator_icc_v1_destroy(image_descr_creator_icc);
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(create_icc_image_description_no_info)
 {
 	struct client *client;
 	struct color_manager cm;
-	struct xx_image_description_creator_icc_v4 *image_descr_creator_icc;
+	struct wp_image_description_creator_icc_v1 *image_descr_creator_icc;
 	struct image_description *image_descr;
 	struct image_description_info *image_descr_info;
 
@@ -952,7 +1014,7 @@ TEST(create_icc_image_description_no_info)
 	color_manager_init(&cm, client);
 
 	image_descr_creator_icc =
-		xx_color_manager_v4_new_icc_creator(cm.manager);
+		wp_color_manager_v1_create_icc_creator(cm.manager);
 
 	/* Create image description based on ICC profile */
 	image_descr = create_icc_based_image_description(&cm, image_descr_creator_icc,
@@ -962,26 +1024,28 @@ TEST(create_icc_image_description_no_info)
 	/* Get image description information, and that should fail. Images
 	 * descriptions that we create do not accept this request. */
 	image_descr_info = image_descr_get_information(image_descr);
-	expect_protocol_error(client, &xx_image_description_v4_interface,
-			      XX_IMAGE_DESCRIPTION_V4_ERROR_NO_INFORMATION);
+	expect_protocol_error(client, &wp_image_description_v1_interface,
+			      WP_IMAGE_DESCRIPTION_V1_ERROR_NO_INFORMATION);
 
 	image_descr_info_destroy(image_descr_info);
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(set_surface_image_description)
 {
 	struct client *client;
 	struct color_manager cm;
-	struct xx_image_description_creator_icc_v4 *image_descr_creator_icc;
+	struct wp_image_description_creator_icc_v1 *image_descr_creator_icc;
 	struct image_description *image_descr;
 
 	client = create_client_and_test_surface(100, 100, 100, 100);
 	color_manager_init(&cm, client);
 
 	image_descr_creator_icc =
-		xx_color_manager_v4_new_icc_creator(cm.manager);
+		wp_color_manager_v1_create_icc_creator(cm.manager);
 
 	/* Create image description based on ICC profile */
 	image_descr = create_icc_based_image_description(&cm, image_descr_creator_icc,
@@ -989,11 +1053,13 @@ TEST(set_surface_image_description)
 	wait_until_image_description_ready(client, image_descr);
 
 	/* Set surface image description */
-	xx_color_management_surface_v4_set_image_description(cm.surface,
-							     image_descr->xx_image_descr,
-							     XX_COLOR_MANAGER_V4_RENDER_INTENT_PERCEPTUAL);
+	wp_color_management_surface_v1_set_image_description(cm.surface,
+							     image_descr->wp_image_descr,
+							     WP_COLOR_MANAGER_V1_RENDER_INTENT_PERCEPTUAL);
 	client_roundtrip(client);
 
 	color_manager_fini(&cm);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
