@@ -27,6 +27,7 @@
 
 #include "weston-test-client-helper.h"
 #include "weston-test-fixture-compositor.h"
+#include "weston-test-assert.h"
 
 static enum test_result_code
 fixture_setup(struct weston_test_harness *harness)
@@ -53,12 +54,10 @@ TEST(drm_smoke) {
 	color_rgb888(&red, 255, 0, 0);
 
 	client = create_client_and_test_surface(0, 0, 200, 200);
-	assert(client);
+	test_assert_ptr_not_null(client);
 
 	surface = client->surface->wl_surface;
-	buffer = create_shm_buffer_a8r8g8b8(client, 200, 200);
-
-	fill_image_with_color(buffer->image, &red);
+	buffer = create_shm_buffer_solid(client, 200, 200, &red);
 
 	for (i = 0; i < 5; i++) {
 		wl_surface_attach(surface, buffer->proxy, 0, 0);
@@ -70,6 +69,8 @@ TEST(drm_smoke) {
 
 	buffer_destroy(buffer);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(drm_screenshot_no_damage) {
@@ -78,7 +79,7 @@ TEST(drm_screenshot_no_damage) {
 	bool ret;
 
 	client = create_client_and_test_surface(0, 0, 200, 200);
-	assert(client);
+	test_assert_ptr_not_null(client);
 
 	/*
 	 * DRM-backend has an optimization to not even call the renderer if
@@ -91,9 +92,11 @@ TEST(drm_screenshot_no_damage) {
 	 */
 	for (i = 0; i < 5; i++) {
 		ret = verify_screen_content(client, "drm_screenshot_no_damage",
-					    0, NULL, i, "Virtual-1");
-		assert(ret);
+					    0, NULL, i, NULL, NO_DECORATIONS);
+		test_assert_true(ret);
 	}
 
 	client_destroy(client);
+
+	return RESULT_OK;
 }
